@@ -400,6 +400,86 @@ class Components:
 
 
     def generate_chart_2(self,df_chart,line_colour):
+        df_signal = df_chart.copy()
+        df_signal = df_signal.dropna(subset=['signal'])
+        df_signal = df_signal.reset_index()
+        signal_open_date = []
+        signal_open_close = []
+        signal_close_date = []
+        signal_close_close = []
+        signal_close_reason = []
+
+
+        for i, row in df_signal.iterrows():
+            if row['signal'] == 'open':
+                signal_open_date.append(df_signal.iloc[i]['date'])
+                signal_open_close.append(df_signal.iloc[i]['close'])
+                signal_close_date.append(df_signal.iloc[i + 1]['date'])
+                signal_close_close.append(df_signal.iloc[i + 1]['close'])
+                signal_close_reason.append(df_signal.iloc[i + 1]['signal'])
+            else:
+                signal_open_date.append(df_signal.iloc[i - 1]['date'])
+                signal_open_close.append(df_signal.iloc[i - 1]['close'])
+                signal_close_date.append(df_signal.iloc[i]['date'])
+                signal_close_close.append(df_signal.iloc[i]['close'])
+                signal_close_reason.append(df_signal.iloc[i]['signal'])
+
+        df_signal['signal_open_date'] = signal_open_date
+        df_signal['signal_open_close'] = signal_open_close
+        df_signal['signal_close_date'] = signal_close_date
+        df_signal['signal_close_close'] = signal_close_close
+        df_signal['close_reason'] = signal_close_reason
+        df_signal = df_signal.set_index('date')
+
+        open_date = []
+        open_close = []
+        close_date = []
+        close_close = []
+        close_reason = []
+        for i, row in df_chart.iterrows():
+            try:
+                open_date.append(df_signal.loc[row['date']].signal_open_date.strftime("%Y-%m-%d"))
+            except:
+                open_date.append(None)
+            try:
+                open_close.append(df_signal.loc[row['date']].signal_open_close)
+            except:
+                open_close.append(None)
+            try:
+                close_date.append(df_signal.loc[row['date']].signal_close_date.strftime("%Y-%m-%d"))
+            except:
+                close_date.append(None)
+            try:
+                close_close.append(df_signal.loc[row['date']].signal_close_close)
+            except:
+                close_close.append(None)
+            try:
+                close_reason.append(df_signal.loc[row['date']].close_reason)
+            except:
+                close_reason.append(None)
+
+        df_signal_final = pd.DataFrame()
+        df_signal_final['open_date'] = open_date
+        df_signal_final['open_close'] = open_close
+        df_signal_final['close_date'] = close_date
+        df_signal_final['close_close'] = close_close
+        df_signal_final['close_reason'] = close_reason
+        df_signal_final['pctchange'] = ((df_signal_final['close_close'] - df_signal_final['open_close']) /
+                                        df_signal_final['open_close']) * 100
+        df_signal_final.open_close = df_signal_final.open_close.round(2)
+        df_signal_final.close_close = df_signal_final.close_close.round(2)
+        df_signal_final.pctchange = df_signal_final.pctchange.round(2)
+
+        hover_template = "<br>".join([
+            "Close Reason: %{customdata[4]}",
+            "Open Date: %{customdata[0]}",
+            "Close Date: %{customdata[2]}",
+            "Open Close: %{customdata[1]}",
+            "Close Close: %{customdata[3]}",
+            "Pct Change: %{customdata[5]}%",
+        ])
+
+
         fig_line = px.line(df_chart, x='date', y="equity_value")
         fig_line.update_traces(line_color='rgba(0, 0, 0, 0)')
         fig_line.update_xaxes(showline=True, zeroline=False, linecolor='white', gridcolor='rgba(0, 0, 0, 0)')
@@ -415,19 +495,19 @@ class Components:
                                font={"color": "white", 'size': 9}, yaxis={'title': 'Equity'},
                                xaxis={'title': ''}
                                )
-        fig_line.add_trace(go.Scatter(mode='markers', hoverinfo='skip',
+        fig_line.add_trace(go.Scatter(mode='markers', customdata=df_signal_final, hovertemplate=hover_template,
                                       x=df_chart['date'], y=df_chart['open'], visible='legendonly',
                                       marker=dict(color='rgba(0, 0, 0, 0)', size=8,
                                                   line=dict(color='yellow', width=1.5)), name='open'), )
-        fig_line.add_trace(go.Scatter(mode='markers', hoverinfo='skip',
+        fig_line.add_trace(go.Scatter(mode='markers', customdata=df_signal_final, hovertemplate=hover_template,
                                       x=df_chart['date'], y=df_chart['close_logic'], visible='legendonly',
                                       marker=dict(color='rgba(0, 0, 0, 0)', size=8,
                                                   line=dict(color='green', width=1.5)), name='close_logic'), )
-        fig_line.add_trace(go.Scatter(mode='markers', hoverinfo='skip',
+        fig_line.add_trace(go.Scatter(mode='markers', customdata=df_signal_final, hovertemplate=hover_template,
                                       x=df_chart['date'], y=df_chart['profit_taking'], visible='legendonly',
                                       marker=dict(color='rgba(0, 0, 0, 0)', size=8,
                                                   line=dict(color='red', width=1.5)), name='profit_taking'), )
-        fig_line.add_trace(go.Scatter(mode='markers', hoverinfo='skip',
+        fig_line.add_trace(go.Scatter(mode='markers', customdata=df_signal_final, hovertemplate=hover_template,
                                       x=df_chart['date'], y=df_chart['stop_loss'], visible='legendonly',
                                       marker=dict(color='rgba(0, 0, 0, 0)', size=8,
                                                   line=dict(color='blue', width=1.5)), name='stop_loss'), )
