@@ -24,10 +24,11 @@ class Plot:
     line_selected = -1
     button_list = (0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
     chart_type = 1
+    relayoutData = []
     init = True
 
 
-    def __new__(self, filename, output_folder, start_date, end_date, para_dict, result_df, generate_filepath):
+    def __new__(self, filename, output_folder, start_date, end_date, para_dict, result_df, generate_filepath, settings):
 
         chart_bg = self.chart_bg
 
@@ -102,6 +103,7 @@ class Plot:
                                   ], style={'margin': '0px 5px'}),
                                   html.Div(id='chart_area',
                                            children=dcc.Graph(id='line_chart', figure=empty_line_chart)),
+
                                   html.Div(id='sub_chart1_area',\
                                            children=dcc.Graph(id='sub_chart1', figure=empty_line_chart,\
                                                               config={'displayModeBar': False}),
@@ -115,7 +117,7 @@ class Plot:
                         html.Div(style={'height': '5px', }),
 
                         html.Div(id='hist_area', children=html.Div(),
-                                 style={'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg}),
+                                 style={'display': 'none'}),
 
                     ]), style={'padding':'0','padding-left':'5px'}, width=9),
                 ])
@@ -139,11 +141,13 @@ class Plot:
             Output('sub_chart2_area', 'style'),
             Output('sub_chart2', 'figure'),
             Output('hist_area', 'children'),
+            Output('hist_area', 'style'),
             Input('sort_method', 'value'),
             Input({'type': 'para-checklist', 'index': ALL}, 'value'),
             State('filter_dropdown_div', 'children'),
             State('filter_input_div', 'children'),
             State('line_chart', 'figure'),
+            Input('line_chart', 'relayoutData'),
             State('filter', 'children'),
             Input('add_button', 'n_clicks'),
             Input('chart1_button', 'n_clicks'),
@@ -156,36 +160,77 @@ class Plot:
             State('sub_chart2_area', 'style'),
             State('sub_chart2', 'figure'),
             State('hist_area', 'children'),
+            State('hist_area', 'style'),
             [Input('button_' + str(i), 'n_clicks') for i in range(10)],
         )
         def display_output(sort_method,para_checklist,filter_dropdown_div,filter_input_div,fig_line,\
-                           filter_div,add_button_clicks,chart1_button_clicks,chart2_button_clicks,\
-                           _add_button_style,filter_name,filter_input_value,sub_chart1_area,sub_chart1,\
-                           sub_chart2_area,sub_chart2,hist_area,*vals):
+                           relayoutData, filter_div,add_button_clicks, \
+                           chart1_button_clicks,chart2_button_clicks,\
+                           _add_button_style,filter_name,filter_input_value,\
+                           sub_chart1_area,sub_chart1,\
+                           sub_chart2_area,sub_chart2,hist_area,hist_style, *vals):
 
             chart1_button_text = ''
             chart2_button_text = ''
 
 
+            if not relayoutData == self.relayoutData:
+                self.relayoutData = relayoutData
+                if self.chart_type == 1:
+                    chart2_button_text = 'Strategy Analysis >'
+                    return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style, \
+                           None, chart1_button_text, chart2_button_text, \
+                           sub_chart1_area, sub_chart1, sub_chart2_area, sub_chart2, hist_area, hist_style
+                else:
+                    # Only trigger when chart 2, change the subchart only
+                    para_key_list = list(para_dict)
+                    para_values = []
+                    for key in para_key_list:
+                        para_values.append(self.graph_df.iloc[self.line_selected][key])
+                    save_path = generate_filepath(filename, output_folder, start_date, end_date, para_dict,
+                                                  para_values)
+                    df = pd.read_csv(save_path)
+                    df_chart = components.prepare_df_chart(df)
+
+                    chart1_button_text = '< Back'
+
+                    sub_chart1 = components.generate_subchart(df_chart, settings['subchart_1'][0],
+                                                              settings['subchart_1'][1], 'yellow')
+                    sub_chart2 = components.generate_subchart(df_chart, settings['subchart_2'][0],
+                                                              settings['subchart_2'][1], '#FF01FE')
+
+                    try:
+                        sub_chart1 = go.Figure(sub_chart1).update_layout(
+                            xaxis_range=[relayoutData['xaxis.range[0]'], relayoutData['xaxis.range[1]']])
+                        sub_chart2 = go.Figure(sub_chart2).update_layout(
+                            xaxis_range=[relayoutData['xaxis.range[0]'], relayoutData['xaxis.range[1]']])
+                    except:
+                        pass
+
+
+                    return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style, \
+                           None, chart1_button_text, chart2_button_text, \
+                           sub_chart1_area, sub_chart1, sub_chart2_area, sub_chart2, hist_area, hist_style
+
+
+
             if chart1_button_clicks:
                 if chart1_button_clicks > self.chart1_button_clicks:
                     self.chart1_button_clicks = chart1_button_clicks
                     self. chart_type = 1
                     sub_chart1_area = {'display': 'none'}
                     sub_chart2_area = {'display': 'none'}
+                    hist_style = {'display': 'none'}
 
 
             if chart2_button_clicks:
                 if chart2_button_clicks > self.chart2_button_clicks:
                     self.chart2_button_clicks = chart2_button_clicks
-                    print('Phase2 Button Pressed')
-                    print(self.line_selected)
                     if self.chart_type == 2:
-                        print('Already Chart 2')
                         chart1_button_text = '< Back'
                         return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style, \
                                None, chart1_button_text, chart2_button_text, \
-                               sub_chart1_area, sub_chart1, sub_chart2_area, sub_chart2, hist_area
+                               sub_chart1_area, sub_chart1, sub_chart2_area, sub_chart2, hist_area, hist_style
                     else:
                         if self.line_selected > -1:
                             para_key_list = list(para_dict)
@@ -204,17 +249,29 @@ class Plot:
 
                             sub_chart1_area = {}
                             sub_chart2_area = {}
-                            sub_chart1 = components.generate_subchart(df_chart, 'volume', 'bar', 'yellow')
-                            sub_chart2 = components.generate_subchart(df_chart, 'sma', 'line', '#FF01FE')
+                            sub_chart1 = components.generate_subchart(df_chart, settings['subchart_1'][0],
+                                                                      settings['subchart_1'][1], 'yellow')
+                            sub_chart2 = components.generate_subchart(df_chart, settings['subchart_2'][0],
+                                                                      settings['subchart_2'][1], '#FF01FE')
 
-                            ### Histograms
-                            period = [3,5,10,30,50]
+
+                            # Histograms
+                            period = settings['histogram_period']
                             # first column of the 5 charts
-                            pct_list = [dbc.Col(html.Div('pct_change',style={'margin-top':'30px','transform':'rotate(-90deg)'}),width=1)]
-                            rise_list = [dbc.Col(width=1)]
-                            fall_list = [dbc.Col(width=1)]
+                            title_list = [dbc.Col(width=1)]
+                            pct_list = [dbc.Col(html.Div('pct_change', style={'font-size': '12px', 'margin-top': '60px',
+                                                                              'margin-left': '40px',
+                                                                              'transform': 'rotate(-90deg)'}), width=1)]
+                            rise_list = [dbc.Col(html.Div('max_rise', style={'font-size': '12px', 'margin-top': '55px',
+                                                                              'margin-left': '40px',
+                                                                              'transform': 'rotate(-90deg)'}), width=1)]
+                            fall_list = [dbc.Col(html.Div('max_fall', style={'font-size': '12px', 'margin-top': '55px',
+                                                                              'margin-left': '40px',
+                                                                              'transform': 'rotate(-90deg)'}), width=1)]
+
 
                             for p in period:
+                                title_list.append(dbc.Col(f'{p} Days',style={'font-size': '12px', 'text-align': 'center'},width=2))
                                 fig_pct, fig_rise, fig_fall = components.generate_histogram(df, p)
                                 pct_list.append( dbc.Col(dcc.Graph(figure=fig_pct,config={'displayModeBar': False})
                                                          ,style={'padding':'0'},width=2))
@@ -223,15 +280,21 @@ class Plot:
                                 fall_list.append(dbc.Col(dcc.Graph(figure=fig_fall,config={'displayModeBar': False})
                                                          ,style={'padding': '0'}, width=2))
 
+
+                            title_list = html.Div(dbc.Row(title_list))
                             pct_list = html.Div(dbc.Row(pct_list))
                             rise_list = html.Div(dbc.Row(rise_list))
                             fall_list = html.Div(dbc.Row(fall_list))
 
-                            hist_area = [pct_list, rise_list, fall_list]
+                            hist_area = [html.Div(style={'height': '5px', }),
+                                         title_list, pct_list, rise_list, fall_list,
+                                         html.Div(style={'height': '5px', })]
+
+                            hist_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg}
 
                             return fig_line, filter_div, filter_dropdown_div, filter_input_div, _add_button_style,\
                                    None, chart1_button_text, chart2_button_text, \
-                                   sub_chart1_area, sub_chart1, sub_chart2_area, sub_chart2, hist_area
+                                   sub_chart1_area, sub_chart1, sub_chart2_area, sub_chart2, hist_area, hist_style
 
 
                         else:
@@ -249,12 +312,14 @@ class Plot:
                    self.chart_type = 1
                    sub_chart1_area = {'display': 'none'}
                    sub_chart2_area = {'display': 'none'}
+                   hist_area = []
+                   hist_style = {'display': 'none'}
                    self.init = True
                    filter_dropdown_div = filter_dropdown
                    filter_input_div = filter_input
                    return fig_line, filter_div, filter_dropdown_div,filter_input_div,_add_button_style, \
                           None, chart1_button_text, chart2_button_text, \
-                          sub_chart1_area, sub_chart1, sub_chart2_area, sub_chart2
+                          sub_chart1_area, sub_chart1, sub_chart2_area, sub_chart2, hist_area, hist_style
 
             ## Add Button Pressed
             if add_button_clicks:
@@ -343,13 +408,12 @@ class Plot:
                 self.chart_type = 1
                 sub_chart1_area = {'display': 'none'}
                 sub_chart2_area = {'display': 'none'}
-
-
+                hist_style = {'display': 'none'}
 
 
             return fig_line, filter_div, filter_dropdown_div,filter_input_div,_add_button_style, \
                    None, chart1_button_text, chart2_button_text, \
-                   sub_chart1_area, sub_chart1, sub_chart2_area, sub_chart2, hist_area
+                   sub_chart1_area, sub_chart1, sub_chart2_area, sub_chart2, hist_area, hist_style
 
 
 
@@ -360,7 +424,6 @@ class Plot:
             State(component_id='performance_matrix', component_property='children'),
         )
         def update_matrix(clickData,performance_matrix):
-            print(clickData)
             button_style = {'text-align': 'right','color':'grey','font-size':'12px'}
             if clickData:
                 if self.chart_type == 1:
